@@ -1,11 +1,6 @@
 from datetime import datetime
-from typing import Any
 
-from requests import Session
-
-from ..devices.gateway import Gateway
 from ..exceptions.device import WrongElementError
-from ..mydevolo import Mydevolo
 from .sensor_property import SensorProperty
 
 
@@ -14,9 +9,6 @@ class MultiLevelSensorProperty(SensorProperty):
     Object for multi level sensors. It stores the multi level sensor state and additional information that help displaying the
     state in the right context.
 
-    :param gateway: Instance of a Gateway object
-    :param session: Instance of a requests.Session object
-    :param mydevolo: Mydevolo instance for talking to the devolo Cloud
     :param element_uid: Element UID, something like devolo.MultiLevelSensor:hdm:ZWave:CBC56091/24#MultilevelSensor(1)
     :key value: Multi level value
     :type value: float
@@ -24,40 +16,50 @@ class MultiLevelSensorProperty(SensorProperty):
     :type unit: int
     """
 
-    def __init__(self, gateway: Gateway, session: Session, mydevolo: Mydevolo, element_uid: str, **kwargs: Any):
+    def __init__(self, element_uid: str, **kwargs):
         if not element_uid.startswith(("devolo.DewpointSensor:",
                                        "devolo.MultiLevelSensor:",
                                        "devolo.ValveTemperatureSensor",
                                        "devolo.VoltageMultiLevelSensor:")):
             raise WrongElementError(f"{element_uid} is not a Multi Level Sensor.")
 
-        super().__init__(gateway=gateway, session=session, mydevolo=mydevolo, element_uid=element_uid, **kwargs)
-
-        self._value = kwargs.get("value", 0.0)
-        self._unit = ""
-        self.unit = kwargs.get("unit", "")
+        super().__init__(element_uid=element_uid, **kwargs)
 
+        self._value = kwargs.pop("value", 0.0)
+        self._unit = kwargs.pop("unit", 0)
 
     @property
     def unit(self) -> str:
         """ Human readable unit of the property. """
-        return self._unit
-
-    @unit.setter
-    def unit(self, unit: int):
-        """ Make the numeric unit human readable, if known. """
-        units = {"dewpoint": {0: "°C", 1: "°F"},
-                 "humidity": {0: "%", 1: "g/m³"},
-                 "light": {0: "%", 1: "lx"},
-                 "temperature": {0: "°C", 1: "°F"},
-                 "Seismic Intensity": {0: ""},
-                 "voltage": {0: "V", 1: "mV"}
-                 }
+        units = {
+            "dewpoint": {
+                0: "°C",
+                1: "°F"
+            },
+            "humidity": {
+                0: "%",
+                1: "g/m³"
+            },
+            "light": {
+                0: "%",
+                1: "lx"
+            },
+            "temperature": {
+                0: "°C",
+                1: "°F"
+            },
+            "Seismic Intensity": {
+                0: ""
+            },
+            "voltage": {
+                0: "V",
+                1: "mV"
+            },
+        }
         try:
-            self._unit = units[self.sensor_type].get(unit, str(unit))
+            return units[self.sensor_type].get(self._unit, str(self._unit))
         except KeyError:
-            self._unit = str(unit)
-        self._logger.debug(f"Unit of {self.element_uid} set to '{self._unit}'.")
+            return str(self._unit)
 
     @property
     def value(self) -> float:
@@ -69,3 +71,4 @@ class MultiLevelSensorProperty(SensorProperty):
         """ Update value of the multilevel sensor and set point in time of the last_activity. """
         self._value = value
         self._last_activity = datetime.now()
+        self._logger.debug("value of element_uid %s set to %s.", self.element_uid, value)
